==> scanbot_sdk-2.10.0.tar.gz: ios/Classes/Classicial Component/DocumentScanner.swift <==
//
//  DocumentScanner.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 27.06.22.
//

import Foundation
import ScanbotSDK

class DocumentScanner: NSObject, SBSDKScanner, SBSDKDocumentScannerViewControllerDelegate, CallErrorHandler  {
    
    private var viewController: SBSDKDocumentScannerViewController?
    
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var waitingForImageSnap: Bool = false
    private var configuration: [String:Any?]
    var shouldRunDetection: Bool = true
    private var detectDocumentAfterSnap: Bool = false
    private var customSnapButton :UIButton? = nil
    
    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String:Any?]
    ){
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view _view: UIView){
        if(!ScanbotSDK.isLicenseValid()){
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                self?.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        customSnapButton = UIButton(frame: CGRect(x: 100, y: 100, width: 100, height: 50))
        
        let rootViewController = UIViewController()
        rootViewController.view = _view
        viewController = SBSDKDocumentScannerViewController(parentViewController: rootViewController,
                                                            parentView: nil, delegate: self)
        viewController?.customSnapButton = customSnapButton
        setDocumentScannerConfiguration(configuration:configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
    }
    
    func attach(view _view: UIView) {
        createNativeView(view: _view)
        detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }
    
    
    func detach(){
        detectorChannel?.setMethodCallHandler(nil)
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController, didSnapDocumentImage documentImage: UIImage, on originalImage: UIImage, with result: SBSDKDocumentDetectorResult, autoSnapped: Bool) {
        print("didSnapDocumentImage")
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        
        guard let uuid = SBSDKUIPageFileStorage.default().add(originalImage) else { return }
        let page : SBSDKUIPage = SBSDKUIPage.init(pageFileID: uuid, polygon: SBSDKPolygon())
        
        
        if(detectDocumentAfterSnap){
            guard let image = page.originalImage() else { return }
            let polygon = DocumentDetector().detectPolygon(on: image)
            page.polygon = polygon
        }
        
        cameraMethodChannel?.invokeMethod("onPictureTaken", arguments: page.encodedString)
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
        
    }
    
    func documentScannerViewControllerShouldDetectDocument(_ controller: SBSDKDocumentScannerViewController) -> Bool {
        return shouldRunDetection && !waitingForImageSnap
    }
    
    func documentScannerViewControllerWillSnapImage(_ controller: SBSDKDocumentScannerViewController) {
        
    }
    
    func documentScannerViewControllerDidFailSnappingImage(_ controller: SBSDKDocumentScannerViewController, withError error: Error?) {
        
    }
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController, configureStatusDetectionLabel label: SBSDKDetectionStatusLabel, forDetectionResult result: SBSDKDocumentDetectorResult) {
        detectorChannel?.invokeMethod("onContourDetected", arguments: result.encodedString)
    }
    
    func setDocumentScannerConfiguration(configuration:[String:Any?]?) {
        applyPolygonConfiguration(configuration)
        
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            viewController?.zoomConfiguration = zoomConfig
        }
        
        applyScannerConfiguration(configuration)
        
    }
    
    fileprivate func applyScannerConfiguration(_ configuration: [String : Any?]?) {
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
            // this is a workaround to make polygon view visible but the all hinsts are not. This need to make flutter native hint component
            viewController?.statusTextConfiguration.textHintBadAngles = ""
            viewController?.statusTextConfiguration.textHintTooDark = ""
            viewController?.statusTextConfiguration.textHintTooNoisy = ""
            viewController?.statusTextConfiguration.textHintTooSmall = ""
            viewController?.statusTextConfiguration.textHintOffCenter = ""
            viewController?.statusTextConfiguration.textHintNothingDetected = ""
            viewController?.statusTextConfiguration.textHintOKManualScan = ""
            viewController?.statusTextConfiguration.textHintOKAutoShutter = ""
            viewController?.statusTextConfiguration.textHintEnergySavingActive = ""
            viewController?.hideDetectionStatusLabel = false
            viewController?.hideSnapButton = true
            if let acceptedSizeScore =  scannerConfiguration["acceptedSizeScore"] as? Double {
                viewController?.acceptedSizeScore = acceptedSizeScore
            }
            
            if let acceptedAngleScore =  scannerConfiguration["acceptedAngleScore"] as? Double {
                viewController?.acceptedAngleScore = acceptedAngleScore
            }
            
            if let ignoreBadAspectRatio = scannerConfiguration["ignoreBadAspectRatio"] as? Bool {
                viewController?.ignoreBadAspectRatio = ignoreBadAspectRatio
            }
            
            if let autoSnappingSensitivity = scannerConfiguration["autoSnappingSensitivity"] as? Double {
                viewController?.autoSnappingSensitivity = autoSnappingSensitivity
            }
            
            if let detectDocumentAfterSnap = scannerConfiguration["detectDocumentAfterSnap"] as? Bool {
                self.detectDocumentAfterSnap = detectDocumentAfterSnap
            }
            
            if let requiredAspectRatios: [String:Any] = configuration?["requiredAspectRatios"] as? [String : Any] {
                var array: [SBSDKAspectRatio] = []
                requiredAspectRatios.forEach { (key: String, value: Any) in
                    if let aspectRatio = SBSDKAspectRatio.init(from: requiredAspectRatios) {
                        array.append(aspectRatio)
                    }
                }
                viewController?.requiredAspectRatios = array
            }
        }
    }
    
    fileprivate func applyPolygonConfiguration(_ configuration: [String : Any?]?) {
        if let scannerConfiguration: [String:Any] = configuration?["contourConfiguration"] as? [String : Any] {
            
            if let autoSnapProgressEnabled = scannerConfiguration["autoSnapProgressEnabled"] as? Bool {
                viewController?.polygonAutoSnapProgressEnabled = autoSnapProgressEnabled
            }
            
            if let strokeWidth =  scannerConfiguration["strokeWidth"] as? Double {
                viewController?.polygonLineWidth = strokeWidth
            }
            
            if let autoSnapProgressStrokeWidth =  scannerConfiguration["autoSnapProgressStrokeWidth"] as? Double {
                viewController?.polygonAutoSnapProgressLineWidth = autoSnapProgressStrokeWidth
            }
            
            if let cornerRadius =  scannerConfiguration["cornerRadius"] as? Double {
                viewController?.polygonCornerRadius = cornerRadius
            }
            
            if let hex = scannerConfiguration["fillOkColor"] as? String {
                viewController?.polygonFillColorAccepted = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["fillColor"] as? String {
                viewController?.polygonFillColorRejected = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["strokeOkColor"] as? String {
                viewController?.polygonLineColorAccepted = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["strokeColor"] as? String {
                viewController?.polygonLineColorRejected = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["autoSnapProgressStrokeColor"] as? String {
                viewController?.polygonAutoSnapProgressColor = UIColor(hexString: hex)
            }
        }
    }
    
    
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        _ = call.arguments as? [String: Any]
        switch call.method {
        case "enable" : shouldRunDetection = true
        case "disable" : shouldRunDetection = false
        case "makeSnap" : customSnapButton?.sendActions(for: .touchUpInside)
        case "setAutoSnappingEnabled": do {
            if(self.viewController?.autoSnappingMode == SBSDKAutosnappingMode.enabled){
                self.viewController?.autoSnappingMode = SBSDKAutosnappingMode.disabled
            } else {
                self.viewController?.autoSnappingMode = SBSDKAutosnappingMode.enabled
            }
        }
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result) //always return some result to the flutter
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}

==> scanbot_sdk-2.11.1.tar.gz: ios/Classes/Classicial Component/DocumentScanner.swift <==
//
//  DocumentScanner.swift
//  scanbot_sdk
//
//  Created by Serhii Chaban on 27.06.22.
//

import Foundation
import ScanbotSDK

class DocumentScanner: NSObject, SBSDKScanner, SBSDKDocumentScannerViewControllerDelegate, CallErrorHandler  {
    
    private var viewController: SBSDKDocumentScannerViewController?
    
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private var waitingForImageSnap: Bool = false
    private var configuration: [String:Any?]
    var shouldRunDetection: Bool = true
    private var detectDocumentAfterSnap: Bool = false
    private var customSnapButton :UIButton? = nil
    
    init(
        cameraMethodChannel: FlutterMethodChannel?,
        detectorChannel: FlutterMethodChannel?,
        finderChannel: FlutterMethodChannel?,
        configuration: [String:Any?]
    ){
        self.cameraMethodChannel = cameraMethodChannel
        self.detectorChannel = detectorChannel
        self.finderChannel = finderChannel
        self.configuration = configuration
    }
    
    func createNativeView(view _view: UIView){
        if(!ScanbotSDK.isLicenseValid()){
            DispatchQueue.main.async { [weak self] in
                guard let callHandler = self?.detectorChannel else {
                    fatalError("callHandler cannot be nil")
                }
                self?.sendLicenseErrorToDetector(callHandler)
            }
            return
        }
        customSnapButton = UIButton(frame: CGRect(x: 100, y: 100, width: 100, height: 50))
        
        let rootViewController = UIViewController()
        rootViewController.view = _view
        viewController = SBSDKDocumentScannerViewController(parentViewController: rootViewController,
                                                            parentView: nil, delegate: self)
        viewController?.customSnapButton = customSnapButton
        let _ = viewController?.view
        setDocumentScannerConfiguration(configuration:configuration)
        cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
    }
    
    func attach(view _view: UIView) {
        createNativeView(view: _view)
        detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
    }
    
    func setFlash(enabled: Bool) {
        viewController?.isFlashLightEnabled = enabled
    }
    
    func isFlashAvailable() -> Bool {
        return viewController?.isFlashLightAvailable ?? false
    }
    
    
    func detach(){
        detectorChannel?.setMethodCallHandler(nil)
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController, didSnapDocumentImage documentImage: UIImage, on originalImage: UIImage, with result: SBSDKDocumentDetectorResult, autoSnapped: Bool) {
        print("didSnapDocumentImage")
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        
        guard let uuid = SBSDKUIPageFileStorage.default().add(originalImage) else { return }
        let page : SBSDKUIPage = SBSDKUIPage.init(pageFileID: uuid, polygon: SBSDKPolygon())
        
        
        if(detectDocumentAfterSnap){
            guard let image = page.originalImage() else { return }
            let polygon = DocumentDetector().detectPolygon(on: image)
            page.polygon = polygon
        }
        
        cameraMethodChannel?.invokeMethod("onPictureTaken", arguments: page.encodedString)
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
        
    }
    
    func documentScannerViewControllerShouldDetectDocument(_ controller: SBSDKDocumentScannerViewController) -> Bool {
        return shouldRunDetection && !waitingForImageSnap
    }
    
    func documentScannerViewControllerWillSnapImage(_ controller: SBSDKDocumentScannerViewController) {
        
    }
    
    func documentScannerViewControllerDidFailSnappingImage(_ controller: SBSDKDocumentScannerViewController, withError error: Error?) {
        
    }
    
    func documentScannerViewController(_ controller: SBSDKDocumentScannerViewController, configureStatusDetectionLabel label: SBSDKDetectionStatusLabel, forDetectionResult result: SBSDKDocumentDetectorResult) {
        detectorChannel?.invokeMethod("onContourDetected", arguments: result.encodedString)
    }
    
    func setDocumentScannerConfiguration(configuration:[String:Any?]?) {
        applyPolygonConfiguration(configuration)
        
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            viewController?.zoomConfiguration = zoomConfig
        }
        
        applyScannerConfiguration(configuration)
        
    }
    
    fileprivate func applyScannerConfiguration(_ configuration: [String : Any?]?) {
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
            // this is a workaround to make polygon view visible but the all hinsts are not. This need to make flutter native hint component
            viewController?.statusTextConfiguration.textHintBadAngles = ""
            viewController?.statusTextConfiguration.textHintTooDark = ""
            viewController?.statusTextConfiguration.textHintTooNoisy = ""
            viewController?.statusTextConfiguration.textHintTooSmall = ""
            viewController?.statusTextConfiguration.textHintOffCenter = ""
            viewController?.statusTextConfiguration.textHintNothingDetected = ""
            viewController?.statusTextConfiguration.textHintOKManualScan = ""
            viewController?.statusTextConfiguration.textHintOKAutoShutter = ""
            viewController?.statusTextConfiguration.textHintEnergySavingActive = ""
            viewController?.statusTextConfiguration.textHintBadAspectRatio = ""
            viewController?.hideSnapButton = true
            if let acceptedSizeScore =  scannerConfiguration["acceptedSizeScore"] as? Double {
                viewController?.acceptedSizeScore = acceptedSizeScore
            }
            
            if let acceptedAngleScore =  scannerConfiguration["acceptedAngleScore"] as? Double {
                viewController?.acceptedAngleScore = acceptedAngleScore
            }
            
            if let ignoreBadAspectRatio = scannerConfiguration["ignoreBadAspectRatio"] as? Bool {
                viewController?.ignoreBadAspectRatio = ignoreBadAspectRatio
            }
            
            if let autoSnappingSensitivity = scannerConfiguration["autoSnappingSensitivity"] as? Double {
                viewController?.autoSnappingSensitivity = autoSnappingSensitivity
            }
            
            if let detectDocumentAfterSnap = scannerConfiguration["detectDocumentAfterSnap"] as? Bool {
                self.detectDocumentAfterSnap = detectDocumentAfterSnap
            }
            
            if let requiredAspectRatios: [String:Any] = configuration?["requiredAspectRatios"] as? [String : Any] {
                var array: [SBSDKAspectRatio] = []
                requiredAspectRatios.forEach { (key: String, value: Any) in
                    if let aspectRatio = SBSDKAspectRatio.init(from: requiredAspectRatios) {
                        array.append(aspectRatio)
                    }
                }
                viewController?.requiredAspectRatios = array
            }
        }
    }
    
    fileprivate func applyPolygonConfiguration(_ configuration: [String : Any?]?) {
        if let scannerConfiguration: [String:Any] = configuration?["contourConfiguration"] as? [String : Any] {
            
            if let autoSnapProgressEnabled = scannerConfiguration["autoSnapProgressEnabled"] as? Bool {
                viewController?.polygonAutoSnapProgressEnabled = autoSnapProgressEnabled
            }
            
            if let strokeWidth =  scannerConfiguration["strokeWidth"] as? Double {
                viewController?.polygonLineWidth = strokeWidth
            }
            
            if let autoSnapProgressStrokeWidth =  scannerConfiguration["autoSnapProgressStrokeWidth"] as? Double {
                viewController?.polygonAutoSnapProgressLineWidth = autoSnapProgressStrokeWidth
            }
            
            if let cornerRadius =  scannerConfiguration["cornerRadius"] as? Double {
                viewController?.polygonCornerRadius = cornerRadius
            }
            
            if let hex = scannerConfiguration["fillOkColor"] as? String {
                viewController?.polygonFillColorAccepted = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["fillColor"] as? String {
                viewController?.polygonFillColorRejected = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["strokeOkColor"] as? String {
                viewController?.polygonLineColorAccepted = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["strokeColor"] as? String {
                viewController?.polygonLineColorRejected = UIColor(hexString: hex)
            }
            
            if let hex = scannerConfiguration["autoSnapProgressStrokeColor"] as? String {
                viewController?.polygonAutoSnapProgressColor = UIColor(hexString: hex)
            }
        }
    }
    
    func stopPreview() {
        viewController?.freezeCamera()
    }
    
    func resumePreview() {
        viewController?.unfreezeCamera()
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        _ = call.arguments as? [String: Any]
        switch call.method {
        case "enable" : shouldRunDetection = true
        case "disable" : shouldRunDetection = false
        case "makeSnap" : customSnapButton?.sendActions(for: .touchUpInside)
        case "setAutoSnappingEnabled": do {
            if(self.viewController?.autoSnappingMode == SBSDKAutosnappingMode.enabled){
                self.viewController?.autoSnappingMode = SBSDKAutosnappingMode.disabled
            } else {
                self.viewController?.autoSnappingMode = SBSDKAutosnappingMode.enabled
            }
        }
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result) //always return some result to the flutter
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}
